package org.ntnu.masteroppg;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

public class Config {
	public static final Integer MEDLINE_XML_PATHS = 1;
	public static final Integer DOC_FIELD_ABSTRACT = 2;
	public static final Integer DOC_FIELD_TITLE = 3;
	public static final Integer DOC_FIELD_PMID = 4;
	public static final Integer NUM_RESULTS = 5;
	public static final Integer GENIA_CHUCKER_PATH = 6;
	public static final Integer EVAL_RES_PATH = 7;
	public static final Integer DOC_FIELD_DOCLEN = 8;
	public static final Integer DEFAULT_INDEX_PATH = 9;
	public static final Integer BM25_INDEX_PATH = 10;
	public static final Integer BM25_SENTENCE_INDEX_PATH = 11;
	public static final Integer DEFAULT_SENTENCE_INDEX_PATH = 12;
	public static final Integer GENETAG_CHUNKER_PATH = 13;
	
	public static final float SENTENCE_BOOST_FACTOR = 2f;
	
	
	private static boolean log4jConfigured = false;

	
	public static String DATADIR = "datadir";
	
	private static boolean initialized = false;
	private static HashMap<Integer, Object> cfgdata = new HashMap<Integer, Object>();
	
	public static Object get(Integer key) {
		if (!initialized)
			loadcfg();

		Object cfgvalue = cfgdata.get(key);
		if (cfgvalue == null)
			Logger.getRootLogger().warn("No configvalue found for: " + key);

		return cfgvalue;
	}
	public static String getStr(Integer key) {
		return (String) get(key);
	}
	public static Integer getInt(Integer key) {
		return (Integer) get(key);
	}
	
	private static void loadcfg() {
		/* TODO?: Put into external config file */
		
		

		/*{ // GENIA
			cfgdata.put(DEFAULT_INDEX_PATH, "/home/dagurval/indexdata");
			cfgdata.put(BM25_INDEX_PATH, "/home/dagurval/bm25indexdata");
		}*/
		{ // GeneTag
			cfgdata.put(DEFAULT_INDEX_PATH, "/home/dagurval/gentag_indexdata");
			cfgdata.put(BM25_INDEX_PATH, "/home/dagurval/gentag_bm25indexdata");	
		}
		
		/*cfgdata.put(DEFAULT_INDEX_PATH, "/home/dagurval/indexdata_tmp");
		cfgdata.put(BM25_INDEX_PATH, "/home/dagurval/indexdata_tmp_bm25");*/

		
		ArrayList<String> medlineXML = new ArrayList<String>();
		medlineXML.add("/home/dagurval/medline/2004/medline_2004_judgement_filtered.xml");
		/*medlineXML.add("/home/dagurval/medline/2004/2004_TREC_XML_MEDLINE_A");
		medlineXML.add("/home/dagurval/medline/2004/2004_TREC_XML_MEDLINE_B1");
		medlineXML.add("/home/dagurval/medline/2004/2004_TREC_XML_MEDLINE_C1");
		medlineXML.add("/home/dagurval/medline/2004/2004_TREC_XML_MEDLINE_D1");
		medlineXML.add("/home/dagurval/medline/2004/2004_TREC_XML_MEDLINE_E1");*/

		
		cfgdata.put(MEDLINE_XML_PATHS, medlineXML);
		
		cfgdata.put(DOC_FIELD_ABSTRACT, "abstract");
		cfgdata.put(DOC_FIELD_TITLE, "title");
		cfgdata.put(DOC_FIELD_PMID, "pmid");
		cfgdata.put(DOC_FIELD_DOCLEN, "doclen");
		
		cfgdata.put(NUM_RESULTS, 1000);

		cfgdata.put(GENETAG_CHUNKER_PATH, "models/ne-en-bio-genetag.HmmChunker");
		cfgdata.put(GENIA_CHUCKER_PATH, "models/ne-en-bio-genia.TokenShapeChunker");
		
		cfgdata.put(DEFAULT_SENTENCE_INDEX_PATH, cfgdata.get(DEFAULT_INDEX_PATH) + "/sentenceIndex");
		cfgdata.put(BM25_SENTENCE_INDEX_PATH, cfgdata.get(BM25_INDEX_PATH) + "/sentenceIndex");
		
		cfgdata.put(EVAL_RES_PATH, "/home/dagurval/evalres");
		
		initialized = true;
	}
	public static void initLog4j() {
		
		if (log4jConfigured) {
			System.err.println("log4j already configured.");
			return;
		}
		try {
			Logger.getRootLogger().addAppender((new FileAppender(
					new PatternLayout("%-5p [%t]: %m%n"), "/tmp/masterlogg", false)));
		} catch (IOException e) {
			e.printStackTrace();
		}
		BasicConfigurator.configure(); 
		log4jConfigured = true;
	}
}
