package org.ntnu.masteroppg;
import java.io.IOException;
import java.util.ArrayList;

import org.apache.log4j.Logger;
import org.apache.lucene.queryParser.ParseException;
import org.ntnu.masteroppg.eval.TrecEval;
import org.ntnu.masteroppg.index.SentenceIndexTest;
import org.ntnu.masteroppg.medline.MedlineIndexSearch;
import org.ntnu.masteroppg.medline.MedlineIndexer;
import org.ntnu.masteroppg.medline.MedlineIndexSearch.SearchAlgorithm;

public class Main {

	private static final String CMD_TREC_EVAL = "trec_eval";
	private static final String CMD_REBUILD = "rebuild";
	private static final String CMD_SEARCH  = "search";
	private static final String CMD_SENTENCE_ITERATE = "sentenceIterate";
	private static Logger log;

	public static void main(String[] args) {
		Config.initLog4j();
		log = Logger.getRootLogger();
		
		if (args.length == 0 || !runCmd(args)) {
			System.err.println("Usage: program rebuild|search|sentenceIterate|trec_eval [query]");
			System.exit(1);
		}

	}

	private static boolean runCmd(String[] args) {
		String cmd = args[0];
		if (cmd.equals(CMD_REBUILD)) {
			log.info("Running " + cmd);
			
			ArrayList<String> arrayList 
				= (ArrayList<String>) Config.get(Config.MEDLINE_XML_PATHS);
			String[] medlineXML = new String[arrayList.size()];
			arrayList.toArray(medlineXML);
			
			for (SearchAlgorithm alg : new SearchAlgorithm[] { SearchAlgorithm.BM25, SearchAlgorithm.LUCENE_DEFAULT }) {
				log.info("Indexing " + alg.name()); 

				try {
					MedlineIndexer indexer = new MedlineIndexer(alg);
					indexer.generateIndex(medlineXML);
				} catch (IOException e) {
					e.printStackTrace();
					System.exit(1);
				}
			}
			
			return true;
		}
		else if (cmd.equals(CMD_SEARCH)) {
			log.info("Running " + cmd);
			if (args.length != 2) {
				log.fatal("No query provided");
				return false;
			}
			
			MedlineIndexSearch search;
			try {
				search = new MedlineIndexSearch(SearchAlgorithm.BM25SENTENCE);
				SearchResults res = search.search(args[1], 
						MedlineIndexSearch.SearchFields.ABSTRACT);
				res.prettyPrint();
				
			} catch (IOException e) {
				e.printStackTrace();
				System.exit(1);
				
			} catch (ParseException e) {
				log.error("Invalid search query");
				System.err.println(e.getMessage());
				return false;
			}
			
			return true;
		}
		
		else if (cmd.equals(CMD_SENTENCE_ITERATE)) {
			log.info("Running " + cmd);
			try {
				SentenceIndexTest.run();
			} catch (Exception e) {
				e.printStackTrace();
				System.exit(1);
			}
			return true;
		}
		else if (cmd.equals(CMD_TREC_EVAL)) {
			TrecEval eval = new TrecEval();
			try {
				eval.run();
			} catch (ParseException e) {
				e.printStackTrace();
				System.exit(1);
			} catch (IOException e) {
				e.printStackTrace();
				System.exit(1);
			}
			return true;
		}
		
		log.fatal("Command " + cmd + " not found.");
		return false;
	}

		
	


}
