package org.ntnu.masteroppg;

import java.io.IOException;
import java.util.Iterator;

import org.apache.lucene.document.Document;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.TopDocs;

public class SearchResults {
	private int totalHits;
	private SearchResult[] results; 
	
	public SearchResults(IndexSearcher searcher, TopDocs docs) throws CorruptIndexException, IOException {
		this.totalHits = docs.totalHits;
		results = new SearchResult[docs.scoreDocs.length];
		

		int i = 0;
		for (ScoreDoc s : docs.scoreDocs) {
			results[i] = new SearchResult(i + 1, searcher.doc(s.doc), s.score);
			i++;
		}
	}

	public void prettyPrint() { 
		for (SearchResult r : results) {
			Document d = r.getDocument();
			String title = d.get(Config.getStr(Config.DOC_FIELD_TITLE));
			//String abstr = d.get(Config.getStr(Config.DOC_FIELD_ABSTRACT));
			String pmdi  = d.get(Config.getStr(Config.DOC_FIELD_PMID));
			
			
			int substrLen = Math.min(50, title.length());
			System.out.println(pmdi + " - " + r.getScore() + " - " + title.substring(0, substrLen) + "... ");
		}
		System.out.println("Total hits: " + totalHits);
	}
	
	public Iterator<SearchResult> iterator() {
		return new Iterator<SearchResult>() {
			private int i = 0;
			public boolean hasNext() {
				return (i < results.length);
			}

			public SearchResult next() {
				return results[i++];
			}

			public void remove() {
				throw new UnsupportedOperationException();
			}
		};
	}
	
	

}
