package org.ntnu.masteroppg.biotracer;

import java.io.IOException;

import javax.sound.midi.SysexMessage;

import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.TermDocs;
import org.apache.lucene.index.TermPositions;
import org.apache.lucene.search.Similarity;
import org.apache.lucene.search.Weight;
import org.ntnu.masteroppg.Config;
import org.ntnu.masteroppg.index.SentenceIndex;
import org.ntnu.masteroppg.luceneExt.SentenceWeights;

public class BM25SentenceTermScorer extends BM25TermScorer {

	private SentenceIndex sindex = null;
	private TermPositions termPos;
	private Term term;

	BM25SentenceTermScorer(Weight weight, TermDocs td, Similarity similarity, IndexReader reader, SentenceIndex sentenceIndex, TermPositions termPos, Term term) {
		super(weight, td, similarity, reader);
		this.termPos = termPos;
		this.term = term;
		sindex = sentenceIndex;

	}
	
	public boolean next() throws IOException {
		if (super.next()) {
			termPos.next();
			return true;
		}
		termPos.close();
		return false;
		
	}

	private float sentenceBoost() {
		try {
			final int ptr = indexReader.document(doc).getField(abstrField).getSentenceIndexPtr();
			final float weight = SentenceWeights.avgTermSentenceWeight(sindex, termPos, ptr);
			return weight;
			
		} catch (CorruptIndexException e) {
			e.printStackTrace();
			System.exit(1);
		} catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		} catch (NullPointerException e) {
			e.printStackTrace();
			System.err.println("Term: " + term);
			System.exit(1);
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(1);
		}
		
		return -1;
	}
	public float score() {
		//return super.score();
		return super.score() + (sentenceBoost() * Config.SENTENCE_BOOST_FACTOR);
		
	}

}
