package org.ntnu.masteroppg.biotracer;

import java.io.IOException;
import java.util.*;

import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.index.Term;
import org.apache.lucene.search.QueryTermVector;
import org.apache.lucene.search.Searcher;
import org.apache.lucene.search.Similarity;


public class CustomSimilarity extends Similarity {
		private float DEFAULT_k3=2f;
		private float k_3=DEFAULT_k3;
		private float queryFactor=1f;
		
		private Map<String, Integer> termDocMap=null;
		
		private double r=0;
		private double R=0;
		private Similarity defsimilarity;
		static public class Model{
			static public final int BM25=1;
			static public final int DFR_BM25=2;
			static public final int LEMUR_TF=3;
			static public final int BM25_URF=4;
			static public final int BM25_SENTENCE = 5;
			
		}
		
		private boolean isPhrase = false;
		
		private int model=Model.BM25;
		
		public CustomSimilarity() {
			super();
			
			this.keyFreqMap = Collections.synchronizedMap(new HashMap<Term, Integer>());
		}
		
		public CustomSimilarity(int m) {
			super();
			
			this.keyFreqMap = Collections.synchronizedMap(new HashMap<Term, Integer>());
			this.model = m;
			//if (m==0) defsimilarity = new DefaultSimilarity();
		}
		
		public CustomSimilarity(Map<String, Integer> tdm, double R, int m) {
			super();
			
			System.out.println("Using URF-based BM25 similarity model");
			this.keyFreqMap = Collections.synchronizedMap(new HashMap<Term, Integer>());
			this.model = m;
			if (m == Model.BM25_URF) {
				this.termDocMap = tdm;
				this.R = R;
			}
			//if (m==0) defsimilarity = new DefaultSimilarity();
		}
		
		
		private Map<Term,Integer> keyFreqMap=null;
		
		 public float queryNorm(float sumOfSquaredWeights) {
			    return 1.0f;
		 }
		 
		 public float lengthNorm(String field, int numTerms) {
	    	    return (float) 1.0f;
	     }
		 
		 public float idf(int docFreq, int numDocs) {
			 if (docFreq==0) return 0f;
			 if (model==Model.BM25 || model== Model.DFR_BM25 || model == Model.BM25_SENTENCE) {
			//	 Idf i = new Idf(); // Using terrier log function
				
				 return (float)Math.log((numDocs - docFreq + 0.5d) / (docFreq + 0.5d));
			 } else if (model == Model.LEMUR_TF) {
				 return (float)Math.log((Math.pow(numDocs/docFreq, 2)));
			 } else if (model == Model.BM25_URF) {
				 float idf = (float)Math.log(((r + 0.5d)*(numDocs-docFreq-R+r+0.5d))/((docFreq-r+0.5d)*(R-r+0.5d)));
				 return idf;
			 }			
			 else {
				 return (float)Math.log(numDocs/docFreq);
			 }
		 }
		 
		 public float tf(float freq) {
			 return freq;
		 }
		 
		 public float keyfactor(Term term) {
			 k_3 = 1000f;
			 Integer keyfreq = keyFreqMap.get(term); 
			 if (keyfreq == null) {
				 keyfreq=1; // for prefix query where each term appears one time only...
			 }
			 
			 switch(model) {
			 case Model.BM25: 
			 case Model.DFR_BM25:
			 case Model.BM25_URF:
			 case Model.BM25_SENTENCE:
				 return ((k_3+1)*keyfreq/((k_3 + keyfreq)));
			 case Model.LEMUR_TF:
				 return keyfreq;
			 
			 }
			return ((k_3+1)*keyfreq/((k_3 + keyfreq)));
		 }
		 
		 public float idf(Term term, Searcher searcher) throws IOException {
			 if (model == Model.BM25_URF) {
				 String sTerm = term.text();
				 if (!isPhrase) 
					 this.r = termDocMap.get(sTerm);
			 }
			 /*if (model == Model.BM25_SENTENCE) 
				 System.err.println("method should not be called with BM25_Sentence");*/
			 
			 return idf(searcher.docFreq(term), searcher.maxDoc())*keyfactor(term); // Modified to include querytermfactor
		 }
		 
		 public int qtf(Term term) {
			 return keyFreqMap.get(term);
		 }
		 		    
		 public float idf(Collection terms, Searcher searcher)  throws IOException {
			float idf = 0.0f;
			
			// Reconstructing the phrase (this is a hack!) to get its r-value
			if (model == Model.BM25_URF) {
				isPhrase = true;
				Term t[] = (Term[])terms.toArray(new Term[0]);
				StringBuffer sb = new StringBuffer(t[0].text());
				for (int i=1; i<t.length; i++) {
					sb.append(" " + t[i].text());
				}
				this.r = termDocMap.get(sb.toString());
			}
			
		    Iterator<Term> i = terms.iterator();
		    while (i.hasNext()) {
		    	Term term =  i.next();
		    	idf += idf(term, searcher);
		    }
		    isPhrase = false;
			 return idf;
		 }
		 
		  /** Implemented as <code>1 / (distance + 1)</code>. */
		  public float sloppyFreq(int distance) {
		    return 1.0f / (distance + 1);
		  }
		    
		  public float coord(int overlap, int maxOverlap) {

			  if (model == Model.BM25_URF) return (float)Math.pow(((overlap) / (float)maxOverlap),  0);//1f;
			  else return (float)Math.pow(((overlap) / (float)maxOverlap),  1);//1f;
	        }

		  public void extractQueryTermFreq(String fields[],  String query) {
			  QueryTermVector qfv = new QueryTermVector(query, new StandardAnalyzer()); 
				
			String t[] = qfv.getTerms();
			int tfreq[] = qfv.getTermFrequencies();
			  for (int i=0; i<fields.length; i++) {
				  extractQueryTermFreq(fields[i], t, tfreq);
			  }
		  }
		  
			public void extractQueryTermFreq(String field,  String terms[], int tfreqs[]) {
				for (int i=0; i<terms.length; i++) {
					this.keyFreqMap.put(new Term(field, terms[i]), tfreqs[i]);
				}
			}
			
			public int usedModel() {
				return model;
			}
			
			public void setModel(int m) {
				model = m;
			}
}