package org.ntnu.masteroppg.index;

import java.util.ArrayList;
import java.util.List;

import com.aliasi.sentences.MedlineSentenceModel;
import com.aliasi.sentences.SentenceModel;
import com.aliasi.tokenizer.IndoEuropeanTokenizerFactory;
import com.aliasi.tokenizer.Tokenizer;
import com.aliasi.tokenizer.TokenizerFactory;

/**
 * Implements SentenceIterator using LangPipe
 */
public class LangPipeSentenceIterator implements SentenceIterator {
    static final TokenizerFactory tokenizerFactory = IndoEuropeanTokenizerFactory.FACTORY;
    static final SentenceModel sentenceModel  = new MedlineSentenceModel();
	private String text;

	public LangPipeSentenceIterator(String text) {
		this.text = text;
	}
	public void run() {
		List<String> tokenList = new ArrayList<String>();
		List<String> whiteList = new ArrayList<String>();
		List<List<Integer>> offsetList = new ArrayList<List<Integer>>();
		Tokenizer tokenizer = tokenizerFactory.tokenizer(text.toCharArray(),0,text.length());
		
		String next;
		while ((next = tokenizer.nextToken()) != null) {
			
			List<Integer> l = new ArrayList<Integer>();
			int tokenStartPos = tokenizer.lastTokenStartPosition();
			l.add(tokenStartPos); // start
			l.add(tokenStartPos + next.length()); // end
			
		    //System.err.println("token : " + next + " start:" + tokenStartPos + " end:" + (tokenStartPos + next.length()));
			
			tokenList.add(next);
			offsetList.add(l);
			whiteList.add(tokenizer.nextWhitespace());
		}
		whiteList.add(tokenizer.nextWhitespace());
		
		int[] sentenceBoundaries = sentenceModel.boundaryIndices(
				tokenList.toArray(new String[0]),
				whiteList.toArray(new String[0])
				);

		if (sentenceBoundaries.length < 1) {
		    //System.err.println("No sentence boundaries found for: " + text);
		    return;
		}
		
		int sentStartTok = 0;
		int sentEndTok = 0;
		
		for (int i = 0; i < sentenceBoundaries.length; ++i) {
		    sentEndTok = sentenceBoundaries[i];
		    final int start = offsetList.get(sentStartTok).get(0);
		    final int end   = offsetList.get(sentEndTok).get(1);
		    StringBuffer strbuff = new StringBuffer((end - start) + 10);
		    
		    for (int j=sentStartTok; j<=sentEndTok; j++) {
		    	strbuff.append(tokenList.get(j) + whiteList.get(j+1));
		    }
		    int tokens = sentEndTok - sentStartTok;
		    sentStartTok = sentEndTok+1;
		    //System.err.println("start: " + start + " end: " + end +  "sentence: " + strbuff.toString());
		    handleNext(strbuff.toString(), start, end, tokens);
		}

	}
	/**
	 * Override this to handle each sentence.
	 */
	public void handleNext(String sentence, int start, int end, int tokens) {
		throw new UnsupportedOperationException();
	}
	
	
}
