package org.ntnu.masteroppg.index;

import java.io.IOException;

import org.apache.log4j.Logger;

import com.aliasi.medline.Abstract;
import com.aliasi.medline.MedlineCitation;
import com.aliasi.medline.MedlineHandler;
import com.aliasi.medline.MedlineParser;

/**
 * For calculating average sentence length in document.
 */
public class SentenceData {
	private long totalLength = 0;
	private int sentenceCount = 0;
	private int maxSentenceLen = -1;
	private String[] medlineXML;
	private Logger log = Logger.getLogger(this.getClass());
	
	
	public long getTotalLength() {
		return totalLength;
	}
	public int getSentenceCount() {
		return sentenceCount;
	}
	public int getMaxSentenceLen() {
		return maxSentenceLen;
	}
	
	public SentenceData(String[] medlineXML) {		
		this.medlineXML = medlineXML;
	}

	
	public float getSentenceAvg() {
		return totalLength / sentenceCount;
	}
	
	public void runMedlineParsing() throws IOException {

		MedlineHandler mh = new MedlineHandler() {
			
			public void delete(String pmid) {
				throw new UnsupportedOperationException();
			}

			public void handle(MedlineCitation citation) {
				Abstract abstrct = citation.article().abstrct();
				String abstctText = abstrct.textWithoutTruncationMarker();
				
				SentenceIterator sitr = new LangPipeSentenceIterator(abstctText) {
					public void handleNext(String sentence, int start, int end, int tokens) {
						sentenceCount++;
						totalLength += tokens;
						maxSentenceLen = Math.max(maxSentenceLen, tokens);
					}
				};
				sitr.run();
				
			}
			
		};
		MedlineParser parser = new MedlineParser(false);
		parser.setHandler(mh);

		for (String xmlPath : medlineXML)
			parser.parse(xmlPath);
		
		log.info("Total length: " + totalLength);
		log.info("Sentence count: " + sentenceCount);
		log.info("Max sentence length: " + maxSentenceLen);
		log.info("Average sentence length: " + getSentenceAvg());
		
	}
	
}
