package org.ntnu.masteroppg.index;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.Random;

import org.apache.log4j.Logger;
import org.ntnu.masteroppg.Config;


public class SentenceIndex implements Serializable {
	private static final long serialVersionUID = 1L;
	
	private ArrayList<ArrayList<Sentence>> index;

	public SentenceIndex() {
		index = new ArrayList<ArrayList<Sentence>>();
	}
	
	/**
	 * Inits room in index for doc, returns id.
	 * 
	 * Not threadsafe.
	 * @return 
	 */
	public int addDoc() {
		index.add(new ArrayList<Sentence>());
		return index.size() - 1;
	}
	
	public void addSentence(int indexId, int offset, float weigth) {
		//System.err.println("index: " + indexId + " offset:" + offset + " weight:" + weight);
		ArrayList<Sentence> docSentences = index.get(indexId);
		docSentences.add(new Sentence(offset, weigth));
	}
	
	private void sort() {
		for (ArrayList<Sentence> sentences : index) {
			Collections.sort(sentences, Sentence.sortByOffset());
		}
	}
	
	public void printDump() {
		sort();
		
		System.out.println("Index size: " + index.size());
		for (int i = 0; i < index.size(); i++) {
			System.out.println("Index record: " + i);
			
			Iterator<Sentence> itr = this.iterator(i);
			while (itr.hasNext()) {
				Sentence s = itr.next();
				System.out.println(
						"\tSentence: " + s.position() 
						+ " Score: "   + s.weight()
				);
			}
		}
	}

	public void store(String path) throws IOException {
		sort();
		SentenceIndexStore.storeIndex(path, this);
	}

	public static SentenceIndex load(String path) throws FileNotFoundException, IOException, ClassNotFoundException {
		return SentenceIndexStore.loadIndex(path);
	}
	
	public Iterator<Sentence> iterator(int indexid) {
		return index.get(indexid).iterator();
	}
	
	public int indexSize() {
		return index.size(); 
	}
	
	
	
}

class SentenceIndexStore {
	private static Logger logger = null;


	public static SentenceIndex loadIndex(String path) throws FileNotFoundException, IOException, ClassNotFoundException {
		log().info("Loading sentence index..");
		ObjectInputStream in = new ObjectInputStream(
				new FileInputStream(new File(path)));
		
		SentenceIndex i = (SentenceIndex) in.readObject();
		log().info("Loaded..");
		return i;
		
	}

	public static void storeIndex(String path, SentenceIndex index) throws IOException  {
		ObjectOutputStream out = null;
		try {
			out = new ObjectOutputStream(
					new FileOutputStream(new File(path)));
		} catch (FileNotFoundException e) {
			log().fatal("'FileNotFound exception when creating a file? Don't know how to handle, exiting.");
			System.exit(1);	
		}
		
		out.writeObject(index);
		log().debug("index stored to " + path);
		
	}
	
	
	private static Logger log() {
		if (logger == null)
			logger = Logger.getLogger(new SentenceIndexStore().getClass());
			
		return logger;
	}
	
	/**
	 * For debugging.
	 */
	public static void main(String args[]) throws FileNotFoundException, IOException, ClassNotFoundException {
		Config.initLog4j();
		SentenceIndex index = new SentenceIndex();
		Random rand = new Random();
		int indexid = -1;
		for (int i = 0; i < 50000; i++) {
			if (indexid == -1 || i % 5 == 0)
				indexid = index.addDoc();
			
			index.addSentence(indexid, rand.nextInt(), rand.nextInt());
		}
		//index.printDump();
		
		SentenceIndexStore.storeIndex("/tmp/index", index);
		index = null;
		
		SentenceIndex index2 = SentenceIndexStore.loadIndex("/tmp/index");
		index2.printDump();
		
	}

}
