package org.ntnu.masteroppg.luceneExt;

import java.io.IOException;

import org.apache.log4j.Logger;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.TermDocs;
import org.apache.lucene.index.TermPositions;
import org.apache.lucene.search.Scorer;
import org.apache.lucene.search.Searcher;
import org.apache.lucene.search.Similarity;
import org.apache.lucene.search.TermQuery;
import org.apache.lucene.search.TermScorer;
import org.apache.lucene.search.Weight;
import org.ntnu.masteroppg.Config;
import org.ntnu.masteroppg.index.SentenceIndex;

public class SentenceTermQuery extends TermQuery {
	private static final long serialVersionUID = 1L;
	Logger log = Logger.getLogger(this.getClass());
	private SentenceIndex sentenceIndex;
	
	
	

	public SentenceTermQuery(Term t, SentenceIndex sentenceIndex) {
		super(t);
		this.sentenceIndex = sentenceIndex;
	}
	protected Weight createWeight(Searcher searcher) throws IOException {
	    return new LocalTermWeight(searcher);
	  }
	
	private class LocalTermWeight extends TermWeight {
		private static final long serialVersionUID = 1376257884089427775L;

		public LocalTermWeight(Searcher searcher) throws IOException {
			super(searcher);
		}
		
		public Scorer scorer(IndexReader reader) throws IOException {
		      TermDocs termDocs = reader.termDocs(term);
		      System.err.println("termDocs: " + termDocs);

		      if (termDocs == null)
		        return null;

		      return new SentenceTermScorer(this, termDocs, similarity,
		                            reader.norms(term.field()), reader, sentenceIndex, term);
		    }
	}
}

class SentenceTermScorer extends TermScorer {

	private String abstrField;
	private IndexReader indexReader;
	private SentenceIndex sindex;
	private TermPositions termPos;
	private Term term;


	public SentenceTermScorer(Weight weight, TermDocs td, Similarity similarity, byte[] norms, IndexReader reader, SentenceIndex sentenceIndex, Term term) {
		super(weight, td, similarity, norms);
		this.indexReader = reader;
		this.abstrField = Config.getStr(Config.DOC_FIELD_ABSTRACT);
		this.sindex = sentenceIndex;
		this.term = term;
		try {
			termPos = indexReader.termPositions(term);
		} catch (IOException e) {
		
			e.printStackTrace();
			System.exit(1);
		}
	}
	
	  public float score() {
		  return super.score() * (sentenceBoost() * Config.SENTENCE_BOOST_FACTOR);
	  }
	  public boolean next() throws IOException {
		  if (super.next()) {
			  if (!termPos.next()) {
				  throw new AssertionError();
			  }
			  return true;
		  }
		  termPos.close();
		  return false;
	  }

	private float sentenceBoost() {
		try {
				final int ptr = indexReader.document(doc).getField(abstrField).getSentenceIndexPtr();
				float weight = -1;
				try {
					weight = SentenceWeights.avgTermSentenceWeight(sindex, termPos, ptr);
				}
				catch (NullPointerException e) {
					
					e.printStackTrace();
					System.err.println("On term: " + term);
					System.exit(1);
				}
				//System.err.println("weight for " + term + ": " + weight);
				return weight;
				
			} catch (CorruptIndexException e) {
				e.printStackTrace();
				System.exit(1);
			} catch (IOException e) {
				e.printStackTrace();
				System.exit(1);
			} catch (Exception e) {
				e.printStackTrace();
				System.exit(1);
			}
			return -1;
		}
}
