package org.ntnu.masteroppg.medline;


import java.io.IOException;
import java.text.BreakIterator;

import org.apache.log4j.Logger;
import org.apache.lucene.index.CorruptIndexException;
import org.ntnu.masteroppg.tools.filter.PMIDRelevantFilter;

import com.aliasi.medline.Abstract;
import com.aliasi.medline.Article;
import com.aliasi.medline.MedlineCitation;
import com.aliasi.medline.MedlineHandler;

public class DocumentHandler implements MedlineHandler {

	
	private int parsed = 0;
	private int skipped = 0;

	private Integer debugParseLimit;
	private boolean limitWarningSent = false;
	private MedlineIndexWriter indexWriter;

	private Integer debugSkip;
	private static Logger log = null;
	//public MedlineToXML mtoxml;
		
	public DocumentHandler(MedlineIndexWriter writer) {
		
		this(writer, null, null);

		
	}
	
	public DocumentHandler(MedlineIndexWriter indexWriter, Integer debugParseLimit, Integer debugSkip) {
		super();
		
		this.indexWriter = indexWriter;
		
		if (log == null) {
			log = Logger.getLogger(this.getClass());
		}
		
		this.debugParseLimit = debugParseLimit;
		this.debugSkip = debugSkip;
		if (debugSkip != null) {
			log.warn("Skipping " + debugSkip + " documents");
		}
	
	}

	public void delete(String pmid) {
		System.err.println("Unhandled 'delete' on " + pmid);
	}
	
	public void handle(MedlineCitation citation) {
		
		if (debugSkip != null) {
			if (skipped <= debugSkip) {
				skipped++;
				return;
			}
		}
		
		if (debugParseLimit != null) {
			if (limitWarningSent)
				return;
			
			if (parsed >= debugParseLimit) {
				log.warn("Parse limit reached, ignoring rest of the citations.");
				limitWarningSent = true;
				return;
			}
		}
		
		String id = citation.pmid();

		
		log.debug("fetching " + id);

		String titleText, abstrText = null;

	    Article article = citation.article();
	    Abstract abstr = article.abstrct();
	    
	    titleText = article.articleTitleText();
	    if (abstr == null) {
	    	log.warn("document " + id + " has no abstract.");
	    	log.warn("Skipping document.");
	    	return;
	    	
	    }
	    else {
	    	abstrText = abstr.textWithoutTruncationMarker();
	    }
	    
	    // if debugging, ignore documents without abstract
	    {
	    	if (debugParseLimit != null && abstrText.equals(""))
	    		return;
	    }
	    parsed++;
	    
	    int docLength = calcDocLength(abstrText);
	    
	    
	    try {
			indexWriter.addDocument(titleText, abstrText, id, docLength);
		} catch (CorruptIndexException e) {
			e.printStackTrace();
			System.exit(1);
		} catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		}
	    
	    
	}

	// hack.. approximate 
	private int calcDocLength(String abstrText) {
		BreakIterator boundary = BreakIterator.getWordInstance();
		boundary.setText(abstrText);

		int start = 0;
		int length = 0;
		for (int end = boundary.next(); end != BreakIterator.DONE; start = end, end = boundary.next()) {
			//System.err.println(abstrText.substring(start, end));
			length++;
			
		}
		return length;
	}


}
