package org.ntnu.masteroppg.medline;

import java.io.IOException;

import org.apache.log4j.Logger;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.store.LockObtainFailedException;
import org.ntnu.masteroppg.index.SentenceData;
import org.ntnu.masteroppg.medline.MedlineIndexSearch.SearchAlgorithm;

import com.aliasi.medline.MedlineParser;

public class MedlineIndexer {
	
	private Logger log = Logger.getLogger(this.getClass());
	private SearchAlgorithm searchAlgorithm;
	private static SentenceData sentenceData = null;
	
	
	public MedlineIndexer(SearchAlgorithm alg) throws IOException {
		this.searchAlgorithm = alg;
	}
	
	public void generateIndex(String[] medlineXML) throws IOException {
		// first full parse to calc avgsentencelen
		
		// Asume we're always working on the same collection for 
		// each run.
		if (sentenceData == null) {
			log.info("Getting avg sentence length");
			sentenceData = getSentenceData(medlineXML);
		}
		else {
			log.info("Sentence data already exists.");
		}

		// then index.
		toIndex(medlineXML, sentenceData);
		


	}

	private SentenceData getSentenceData(String[] medlineXML) throws IOException {
		SentenceData data = new SentenceData(medlineXML);
		data.runMedlineParsing();
		
		return data;
	}

	private void toIndex(String[] medlineXML, SentenceData sentenceData) throws CorruptIndexException, LockObtainFailedException, IOException {
		MedlineParser parser = new MedlineParser(false);
		MedlineIndexWriter indexWriter 
			= new MedlineIndexWriter(sentenceData, searchAlgorithm);
		
		//DocumentHandler dochandler = new DocumentHandler(indexWriter, 10, 24260 - 2);
		//DocumentHandler dochandler = new DocumentHandler(indexWriter, 3, 24260 - 5);
		DocumentHandler dochandler = new DocumentHandler(indexWriter, null, null);
		parser.setHandler(dochandler);
		
		for (String xmlPath : medlineXML) {
			parser.parse(xmlPath);
		}
		indexWriter.close();
	}
}

