package org.ntnu.masteroppg.tools.filter;

import java.io.IOException;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;

import com.aliasi.medline.MedlineCitation;
import com.aliasi.medline.MedlineHandler;
import com.aliasi.medline.MedlineParser;

public class FilterQrelsToXML {
	private static final String QRELS_PATH 
		//= "/home/dagurval/medline/2004/04.qrels.txt";
		= "/home/dagurval/m/eval/04.judgments.txt";
	private static final String OUT_FILE 
		= "/home/dagurval/medline_2004_judgement_filtered.xml";
	private static final String[] MEDLINE_XML = {
		"/home/dagurval/medline/2004/2004_TREC_XML_MEDLINE_A",
		"/home/dagurval/medline/2004/2004_TREC_XML_MEDLINE_B1",
		"/home/dagurval/medline/2004/2004_TREC_XML_MEDLINE_C1",
		"/home/dagurval/medline/2004/2004_TREC_XML_MEDLINE_D1",
		"/home/dagurval/medline/2004/2004_TREC_XML_MEDLINE_E1"
	};
	
	public static void main(String args[]) throws IOException {
		BasicConfigurator.configure();
		CitationHandler handler = new CitationHandler(
				new PMIDRelevantFilter(
						QRELS_PATH, PMIDRelevantFilter.Regex.JUDGEMENT),
				OUT_FILE
				
		);
		MedlineParser parser = new MedlineParser(true);
		parser.setHandler(handler);
		
		
		
		for (String xmlPath : MEDLINE_XML) {
			parser.parse(xmlPath);
		}

		handler.close();
	}
}

class CitationHandler implements MedlineHandler {
	
	private PMIDRelevantFilter filter;
	private int filtered = 0;
	private MedlineToXML mtoxml = null;
	private Logger log = Logger.getLogger(this.getClass());

	public CitationHandler(PMIDRelevantFilter filter, String outPath) {
		this.filter = filter;
		try {
			mtoxml = new MedlineToXML(outPath);
		} catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		}
	}

	public void close() throws IOException {
		log.info("Filtered " + filtered);
		mtoxml.close();
		
	}

	public void delete(String pmid) {
		throw new UnsupportedOperationException();
		
	}

	public void handle(MedlineCitation citation) {
		String id = citation.pmid();
		
		if (filter != null && filter.isFiltered(id)) {
			filtered++;
			return;
		}
		
		try {
			mtoxml.appendCitation(citation);
		} catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		}
	
		
	}
	
}
