package org.ntnu.masteroppg.tools.filter;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.apache.log4j.Logger;

import com.aliasi.medline.MedlineCitation;

public class MedlineToXML {
	
	private Logger log;
	private BufferedWriter writer;
	private boolean inclRawXMLcheck = false;
	private int citationsAppended = 0; 
	
	
	public MedlineToXML(String xmlStorePath) throws IOException {
		log = Logger.getLogger(this.getClass());
		log.info("Opening " + xmlStorePath + " for writing.");
		File f = new File(xmlStorePath);
		
		writer = new BufferedWriter(
				new FileWriter(f));
		
		writeXMLheader();
		
	}
	
	public void appendCitation(MedlineCitation c) throws IOException {

		if (!inclRawXMLcheck) {
			inclRawXMLcheck = true;
			if (c.xmlString().equals("null"))
				throw new AssertionError("MedlineParser does not include raw XML. (Use 'new MedlineParser(true)')");

		}
		writer.append(c.xmlString());
		citationsAppended++;
		log.info("Appended citation " + c.pmid() 
				+ ". Appended: " + citationsAppended);
	}
	
	public void close() throws IOException {
		writeXMLfooter();
		writer.close();
	}
	
	private void writeXMLheader() throws IOException {
		writer.append("<!DOCTYPE MedlineCitationSet\n"
				+ "PUBLIC \"-//NLM//DTD NLM Medline, 1st November 2003//EN\"\n"
				+ "\"http://www.nlm.nih.gov/databases/dtd/nlmmedline_031101.dtd\">\n"
				+ "<MedlineCitationSet>\n");
	}
	
	private void writeXMLfooter() throws IOException {
		writer.append("</MedlineCitationSet>\n");
	}
	

}
