package org.ntnu.masteroppg.tools.filter;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

/**
 * Filter out documents that have no relevance jusdgement.
 * 
 * @author dagurval
 */

public class PMIDRelevantFilter {
	private String qrelsPath;
	
	private static Logger log;
	private static Pattern idPattern = null;

	public static enum Regex { QRELS, JUDGEMENT };
	private HashMap<String, Boolean> pmidKeepList;
	
	public PMIDRelevantFilter(String qrelsPath, Regex regex) throws IOException {
		this.qrelsPath = qrelsPath;
		if (log == null)
			log = Logger.getLogger(this.getClass());
		
		if (idPattern == null) {
			switch (regex) {
			case QRELS:
				idPattern = Pattern.compile("^\\s*\\d+\\s+\\d+\\s+(\\d+)", 
				Pattern.DOTALL | Pattern.CANON_EQ);
				break;
			case JUDGEMENT:
				idPattern = Pattern.compile("^\\s*\\d+\\s+(\\d+)", 
				Pattern.DOTALL | Pattern.CANON_EQ);
				break;
			default:
				throw new UnsupportedOperationException("Unknown regex");
			}
		}
		
		loadFiltered();
	}
	
	public void loadFiltered() throws IOException {
		log.info("Loading pmid filterlist from " + qrelsPath);
		HashMap<String, Boolean> pmids = new HashMap<String, Boolean>();
		
		File file = new File(qrelsPath);
		BufferedReader reader = new BufferedReader(
				new InputStreamReader(new FileInputStream(file)));
		
		String line;
		while ((line = reader.readLine()) != null) {
			String id = readID(line);
			if (id == null)
				continue;
			
			pmids.put(id, true);
		//	log.debug(id);
		}
		log.info("Loaded " + pmids.size() + " pmids into filter.");
		this.pmidKeepList = pmids;
		
	}
	
	public boolean isFiltered(String pmid) {
		return !pmidKeepList.containsKey(pmid);
	}

	private String readID(String line) {
		Matcher idMatcher = idPattern.matcher(line);
		if (idMatcher.find()) {
			return idMatcher.group(1);
		}
		log.warn("no id match for line: " + line);
		return null;
	}
}
