package org.ntnu.masteroppg.tools.queryextract;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Set;

import org.ntnu.masteroppg.Config;

import com.aliasi.chunk.Chunk;
import com.aliasi.chunk.Chunker;
import com.aliasi.chunk.Chunking;
import com.aliasi.util.AbstractExternalizable;

public class TopicQueryExtract {
	private static final int NUM_TOPICS = 50;
	private static Chunker chunker;
	public static void main(String[] args) {
		
		File genia = new File(Config.getStr(Config.GENIA_CHUCKER_PATH));
		try {
			chunker = (Chunker) AbstractExternalizable.readObject(genia);
		} catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			System.exit(1);
		}
		
		ArrayList<String[]> topicTerms = new ArrayList<String[]>();
		
		int topic = 1;
		for (int i = 0; i < NUM_TOPICS; i++) {
			String[] chunks = chuckStr(TOPIC_NEEDS[i]);
			
			if (chunks.length == 0) {
				System.err.println("no terms for topic " + topic 
						+ ", chucking context.");
				chunks = chuckStr(TOPIC_CONTEXT[i]);
			}
			topic++;
			topicTerms.add(chunks);
		}
		prettyPrint(topicTerms);
		javaPrint(topicTerms);
		
	}



	private static void javaPrint(ArrayList<String[]> topicTerms) {
		System.out.println("TOPIC_QUERIES = {");
		for (String[] terms : topicTerms) {
			System.out.print("\"");
			for (String t : terms) {
				System.out.print(t + " ");
			}
			System.out.println("\",");
		}
		System.out.println("};");
	}



	private static void prettyPrint(ArrayList<String[]> topicTerms) {
		int i = 1;
		for (String[] terms : topicTerms) {
			System.out.print("Topic " + i + ": ");
			for (String t : terms) {
				System.out.print(t + ", ");
			}
			System.out.println();
			i++;
		}
	}


	
	private static String[] chuckStr(String str) {
		Chunking c = chunker.chunk(str);
		ArrayList<String> chunks = new ArrayList<String>();
		
		//System.out.println("input=" + s +
			//	"\tchunking=" + c);


		Set<Chunk> set = c.chunkSet();
		Iterator<Chunk> it = set.iterator();
		boolean hasOnlyOther = true;
		while (it.hasNext()) {
			if (!it.next().type().equals("other_name")) {
				hasOnlyOther = false;
				break;
			}
		}
		
		it = set.iterator();
		while (it.hasNext()) {
	        Chunk chunk = it.next();
	        
	        int start = chunk.start();
	        int end = chunk.end();
	        //System.err.println(chunk);
	        String text = str.substring(start,end);
	        if (!hasOnlyOther && chunk.type().equals("other_name")) {
	        	System.err.println("Ignoring chunk " + " " + text + " " + chunk);
	        	continue;
	        }
	           //System.out.println("     chunk=" + chunk + "  text=" + text);
	           //System.out.println("Matched type: " + chunk.type());
	           //s.addMatch(chunk.type());
	        chunks.add(text);
		}
		
		String[] strArray = new String[chunks.size()];
		chunks.toArray(strArray);
		return strArray;
	}
	
	private static final String[] TOPIC_CONTEXT = {
		"Ferroportin1 (also known as SLC40A1; Ferroportin 1; FPN1; HFE4; IREG1; Iron regulated gene 1; Iron-regulated transporter 1; MTP1; SLC11A3; and Solute carrier family 11 (proton-coupled divalent metal ion transporters), member 3) may play a role in iron transport.",               
		"Determine protocols to generate transgenic mice having a single copy of the gene of interest at a specific location.",                     
		"Relevant articles describe genes involved in kidney development.",                                                                         
		" What genes are expressed only in the mouse kidney and not in other tissues?",                                                             
		" Laboratory preparations can be enriched for certain kinds of proteins if the cellular compartment in which they reside is purified away from the rest of the cell contents.",                                                                                                         
		"There are many genes involved in Fanconi Anemia and the downstream pathways of FancD2 in flies. The FancD2 is monoubiquitylated and there are 2 components of the FancD2 pathway. The researcher studies the FancD2 pathway in flies.",                                                
		"Researcher is interested in how oxidative stress effects DNA repair.",                                                                     
		" Are there genes and mechanisms that are utilized by more than one of these fields? A relevant article mentions a gene or pathway, DNA repair, and one or more oxidative or cancerous diseases.",                                                                                                                                          
		"mutY is particularly challenging, because it is also known as hMYH. This is further complicated by the fact that myoglobin genes are also typically located in search results.",                                                                                                       
		"Interested in role that NEIL1 plays in DNA repair.",                                                                                       
		"Researching genes and proteins (pathways) common to DNA repair, oxidative diseases, skin-carcinogenesis, and UV-carcinogenesis.",          
		"Project is to characterize Smad4 knockout mouse in skin (specifically skin) to establish signaling network. Identify all Smad4 targets to compare gene expression patterns of the knockout mouse to the normal mouse.",                                                                
		"TGFB plays a crucial role in regulating angiogenesis, a biological process that occurs during development and homeostasis, as well as during inflammatory perturbation.",                                                                                                              
		"The laboratory wants to identify components of the the TGFB signaling pathway in HNSCC, and determine new targets to study HNSCC.",        
		"The laboratory wants to know more about the role of ATPases in apoptosis.",                                                                
		"A relevant document is one that discusses protein interactions interactions involving members of the AAA protein family that can help to determine their functional importance.",                                                                                                      
		"One aspect of determining how an antibody works is to determine its binding affinity. A relevant document is one which discusses the binding affinity of DO1.",                                                                                                                        
		"It is possible that Gis4 plays a role between cell cycle and yeast carbon pathways and that there is a link between cell cycle and metabolism. A relevant document is one that supports or refutes this hypothesis with regard to the properties of Gis4 in one or both processes.",
		"Gis4 may play a role between the cell cycle and yeast carbon pathways. SNF1 is an upstream kinase of Gis 4.",
		"Ubiquitin and ubiquitin-like proteins have important roles in controlling cell division, signal transduction, embryonic development, endocytic trafficking, and the immune response.",
		"DNA damage may cause cell cycle arrest or apoptosis. p63 and p73 may play a role in mediating these sequelae of DNA damage.",
		"DNA damage may cause cell cycle arrest or apoptosis. p53 plays a role in mediating these sequelae of DNA damage.",
		"The researcher identified a protein in another yeast species and wants to compare it to the same one in Saccharomyces cerevisiae.",
		"A research group is preparing a manuscript about four poorly characterized mouse PGRP genes. Their findings include new information about gene regulation. They report longer DNA and protein sequences than those found in GenBank, and sub-cellular location discrepancies.",
		"New information about experiments and genes involved in scleroderma.",
		"Information gathering for the purpose of supplementing the information from a local protocol.",
		"New information about experiments and genes involved in autophagic cell death.",
		"Collection of information regarding the potential relationship between apoptosis and autophagy.",
		"The laboratory has isolated some gyrA mutations in E. coli. They want to compare their mutant gyrA with the wild-type and other mutant sequences.",
		"The laboratory needs markers to follow Nkx family-member expression and activity.",
		"TOR is a serine-threonine kinase in a pathway involved in the control of cell growth and proliferation, and it is the target of the signaling inhibitor rapamycin.",
		"A xenograft animal model of cancer is one in which foreign tumor tissue is grafted into animals, usually rodents, providing a means to test various compounds for their ability to slow or halt tumor growth.",
		"The ultimate goal of this initial research study, is to identify mouse genes that will influence the outcome of blood borne pathogen infections.",
		"Information needed to contribute to the development of a standardized annotated database of Cryptococcus neoformans genome.",
		"Need to understand the variety of functions that involve this domain.",
		"Further information about a gene is needed after it is identified through a gene expression profile. The genes are related to synaptic plasticity in learning and memory.",
		"Need to put specific PAM research in the context of other researchers work.",
		"Candidate gene testing within a large Scottish case-control study of genetic risk factors for stroke. Future research includes investigations into other ethnically distinct populations.",
		"A relevant document is one which discusses genes that could be considered as candidates to test in a randomized controlled trial which studies the genetic risk factors for stroke.",
		"Information gathering to design assays to determine the nature of donor cells in tissues of chimaeric animals.",
		"Comparing protein mutations detected utilizing mass spectrometry.",
		"Information is required on the disruption of functions from genomic DNA rearrangements.",
		"A relevant document is one that discusses studies on Sleeping Beauty. Interviewee's group studies a related element and want to know what others are doing in a similar field.",
		"Need to identify genes that are most likely to be involved in the nerve growth factor pathway.",
		"Want to identify genes involved in mental disorders.",
		"After being identified via microarrays, the biological processes the genes are involved in needs to be discovered.",
		"Early research goals included learning more about BCL2-interacting molecules, which facilitated identifying new inhibitors during preliminary testing.",
		"The interviewee wished to determine the interests and focus of a fellow scientist that was investigating similar topics to their own.",
		"A DNA sequence isolated in the laboratory is often sequenced only partially, until enough sequence is generated to identify the gene. In these situations, the rest of the sequence is inferred from matching clones in the public domain. When there is difficulty in the laboratory manipulating the DNA segment using sequence-dependent methods, the laboratory isolate must be re-examined.",
		"The researcher is not satisfied with the yield of expressing a protein in E. coli when grown at low temperature and is searching for a better solution. The researcher is willing to try a different organism and/or method."
	
	};
	
		private static final String[] TOPIC_NEEDS = {                                           
		"Find articles about Ferroportin-1, an iron transporter, in humans.",                                                                       
		"Find protocols for generating transgenic mice.",                                                                                           
		"What is the time course of gene expression in the murine developing kidney?",                                                              
		"What mouse genes are specific to the kidney?",                                                                                             
		" Articles are relevant if they describe methods for subcellular fractionation of nuclei.",
		"Find articles about function of FancD2.",
		"Find correlation between DNA repair pathways and oxidative stress.",
		"Genes and proteins (pathways) common to DNA repair, oxidative diseases, skin-carcinogenesis, and UV-carcinogenesis.",
		"Find articles about the function of mutY in humans.",
		"Find articles about the role of NEIL1 in repair of DNA.",
		"Find articles regarding carcinogenesis induced in hairless mice.",
		" Find articles describing genes that are regulated by the signal transducing molecule Smad4.",
		"Documents regarding the role of TGFB in angiogenesis in skin with respect to homeostasis and development.",
		"Documents regarding TGFB expression or regulation in HNSCC cancers.",
		"Find information on role of ATPases in apoptosis",
		"How do AAA proteins mediate interaction with lipids or DNA and what is their functional impact?",
		"Determine binding affinity of anti-p53 monoclonal antibody DO1.",
		"Properties of Gis4 with respect to cell cycle and/or metabolism.",
		"What similarities and differences exist between the upstream promoter regions of GAL1 and SUC1? Are there co-repressors or co-activators? If so, are they regulated by SNF1?",
		" Which biological processes are regulated by having constituent proteins modified by covalent attachment to ubiquitin or ubiquitin-like proteins?",
		"Do p63 and p73 cause cell cycle arrest or apoptosis related to DNA damage?",
		"Does p53 respond differently to different DNA-damaging agents? Do they respond differently to single-strand versus double-strand breaks?",
		"Which Saccharomyces cerevisiae proteins are involved in the ubiquitin proteolytic pathway?",
		"Find all reports describing mouse peptidoglycan recognition proteins (PGRP).",
		"Identify studies that include genome-wide scans and microarray analysis in the investigation of scleroderma.",
		"Retrieval of information regarding the role of BUB2 and BFA1 in cytokinesis in yeast.",
		"Experiments establishing positive or negative interconnection between autophagy and apoptosis.",
		"Studies that investigate similarities in morphological changes among apoptosis and autophagy processes.",
		"Documents containing the sequences and phenotypes of E. coli gyrA mutations.",
		"Documents identifying genes regulated by Nkx gene family members.",
		"Reports that provide possible links between neurofibromatosis and TOR signaling.",
		"Find reports that describe xenograft models of human cancers.",
		"Identify research on mutant mouse strains and factors which increase susceptibility to infection by Histoplasma capsulatum.",
		"Articles reporting experiments allowing annotation of gene products of Cryptococcus.",
		"What is the function of proteins containing WD40 repeats?",
		"Background information on RAB3A.",
		"What research is being done on peptide amidating enzyme, PAM?",
		"Information concerning genetic loci that are associated with increased risk of stroke, such as apolipoprotein E4 or factor V mutations.",
		"Identify genes as potential genetic risk factors candidates for causing hypertension.",
		"To identify the antigens expressed by lung epithelial cells and the antibodies available.",
		"What phenotypes have been described resulting from mutations in the Cystic Fibrosis conductance regulator gene?",
		"What genes show altered behavior due to chromosomal rearrangements?",
		"Studies of Sleeping Beauty transposons.",
		"Create a list of all the nerve growth factor pathway proteins.",
		"What genetic loci, such as Mental Health Wellness 1 (MWH1) are implicated in mental health?",
		"What human biological processes is RSK2 known to be involved in?",
		"Research the human gene BCL-2 to determine if there are antagonists and inhibitors inside of a cell.",
		"What is the focus of studies involving the members of the human UNC gene family?",
		"Find reports and glyphosate tolerance gene sequences in the literature.",
		"Find research on improving protein expressions at low temperature in Escherichia coli bacteria."
	};

}
