#!/usr/bin/env perl
use strict;
use warnings;
use Readonly;
use Data::Dumper;
#use CGI::Carp qw(fatalsToBrowser);
use CGI;
use Carp;
use Template;
use Time::Duration;
use treceval qw(eval_run chart);
use JSON::XS;

Readonly::Scalar my $TREC_EVAL_CMD => "/home/dagurval/master/tools/trec_eval";
Readonly::Scalar my $EVAL_RES_DIR => "/home/dagurval/evalres";
Readonly::Scalar my $RESFILE_REGEX => qr{^(.*?)_results_(\d+)$};

if (CGI::param('chart')) {
	my $alg1 = CGI::param('alg1');
	my $alg2 = CGI::param('alg2');
	my $time1 = CGI::param('time1');
	my $time2 = CGI::param('time2');
	my $num_docs = CGI::param('num_docs');
	print CGI::header('image/png');
	chart(
		$alg1, [ eval_run($alg1, $time1, 1, $num_docs) ],
		$alg2, [ eval_run($alg2, $time2, 1, $num_docs) ]
	);
	exit;
}




if (CGI::param('graph')) {
	my $time = CGI::param('time');
	my $algo = CGI::param('algo');
	my $full = CGI::param('full') ? 1 : 0;
	my $num_docs = CGI::param('num_docs');
	my $json = JSON::XS->new;
	$json->pretty(1);
	print CGI::header("text/plain");
	print $json->encode([eval_run($algo, $time, $full, $num_docs)]);
	exit;
}

sub list_results {
	opendir my $dirh, $EVAL_RES_DIR
		or croak "res dir:", $!;

	my %results;
	for my $file (readdir $dirh) {
		next if $file eq "." || $file eq "..";
		unless ($file =~ $RESFILE_REGEX) {
			warn "ignoring file $file";
			next;
		}
		my ($algoritm, $run) = ($1, $2);
		push @{$results{$algoritm}}, $run;
	}
	for my $alg (keys %results) {
		my @sorted = sort { $a <=> $b } @{$results{$alg}};
		if (@sorted > 10) {
			$results{$alg} = [ splice @sorted, -10 ];
		}
	} 

	return %results;
}

my $tpl = Template->new(INCLUDE_PATH => "tpl", DEBUG => 1, FILTERS => {
	ago => sub { ago(time() - $_[0], 1) },
});
print CGI::header(-type => "text/html", -charset => "UTF-8");
$tpl->process("index.tpl", { results => { list_results() } }) or croak $tpl->error;

1;
