package treceval;
use strict;
use warnings;
use Readonly;
use Carp;
use Data::Dumper;
use Exporter qw(import);
use JSON::XS;
use Chart::Gnuplot;
our @EXPORT = qw(eval_run chart);

Readonly::Scalar my $TREC_EVAL_CMD => "/home/dagurval/master/tools/trec_eval";
Readonly::Scalar my $QRELS => "/home/dagurval/medline/2004/04.qrels.txt";
Readonly::Scalar my $EVAL_RES_DIR => "/home/dagurval/evalres";

sub eval_run {
	my ($alg, $time, $full, $num_docs) = @_;
	croak "invalid algorithm"
		unless $alg =~ /^\w+$/;
	croak "invalid time"
		unless $time =~ /^\d+$/;
	my @eval_data;

	croak "invalid numdocs attr"
		unless $num_docs =~ /^\d+$/;
	$num_docs = "-M$num_docs";
	
	my $res_path = "$EVAL_RES_DIR" . "/" . $alg . "_results_" . $time;
	my $full_cmd = $full ? " -q " : "";

	my $cmd = "$TREC_EVAL_CMD $num_docs $full_cmd $QRELS $res_path ";
	warn "trec_eval command: $cmd";
	my $data = `$cmd`;
	for my $line (split "\n", $data) {
		unless ($line =~ /^((?:\w|-|\.)+)\s+(.*?)\s+(\d*\.?\d*)/) {
			warn "ignoring line '$line'";
		}
		push @eval_data, [$1, $2, $3];
	}
	return @eval_data;
}

sub chart {
	my ($alg1, $dataset_1_ref, $alg2, $dataset_2_ref) = @_;

	my $file = "/tmp/plot" . int(rand(1000))  . ".png";
	my $chart = Chart::Gnuplot->new(
		output => $file,
		title => "Comparison of $alg1 and $alg2",
		xtics => 1,
		#imagesize => "720,640",
		#xlabel => "x",
		#ylabel => "y",
	);
	$chart->plot2d(
		chartify_dataset($dataset_1_ref, $alg1), 
		chartify_dataset($dataset_2_ref, $alg2)
	);
	open my $fh, $file 
		or croak "opening graph file $file", $!;
	print join q{}, <$fh>;
	unlink $file;
	1;
}

sub chartify_dataset {
	my ($dataset_ref, $title) = @_;
	#my %set = (x => [], y => []);
	my @points;
	my $i = 1;
	for my $d (@{$dataset_ref}) {
		my ($x, $query, $res) = @{$d};
		next unless $x eq "map";
		next if $query eq "all";
		push @points, [ $query, $res ];
		#push @{$set{x}}, "$x ($query)";
		#push @{$set{y}}, $res;
	}
	#croak Dumper(\@points);
	
	#croak Dumper(\%set);
	return Chart::Gnuplot::DataSet->new(
		#xdata => $set{x},
		#ydata => $set{y},
		width => 2,
		points => \@points,
		title => $title,
		style => "lines",
	);
}
#
#print Dumper([eval_run("SENTENCE", "1240317670")]);
#my $alg1 = "BM25SENTENCE";
#my $alg2 = "BM25";
#my $timestamp = "1242384318";
#print chart(
#	$alg1, [ eval_run($alg1, $timestamp, 1) ],
#	$alg2, [ eval_run($alg2, $timestamp, 1) ]
#);

1;
